﻿using BPA.Franchisee.Application.FranchiseeCenter.GoodsServices;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Goods;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsAttribute;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.DataBase
{
    [ApiDescriptionSettings("Goods", Tag = "商品管理")]
    public class GoodsServices: IDynamicApiController, ITransient
    {
        IGoodsService _goodsService;
        public GoodsServices(IGoodsService goodsService)
        {
            _goodsService=goodsService;
        }
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/page")]
        public async Task<PageUtil> GetGoodsPage(GoodsQueryDto dto)
        {
            return await _goodsService.GetGoodsPage(dto);
        }
        /// <summary>
        /// 添加商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/add")]
        public async Task<bool> AddGoods(GoodsDto dto)
        {
            return await _goodsService.AddGoods(dto);
        }
        /// <summary>
        /// 更新商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/update")]
        public async Task<bool> UpdateGoods(GoodsDto dto)
        {
            return await _goodsService.UpdateGoods(dto);
        }
        /// <summary>
        /// 删除商品
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("/api/goods/delete")]
        public async Task<bool> DeleteGoods(string id)
        {
            return await _goodsService.DeleteGoods(id);
        }
        /// <summary>
        /// 查询商品单位
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/goods/getgoodsuintlist")]
        public async Task<List<dynamic>> GetGoodsUintList()
        {
            return await _goodsService.GetGoodsUintList();
        }
        /// <summary>
        /// 添加商品单位
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/addgoodsuint")]
        public async Task<bool> AddGoodsUint(GoodsUintDto dto)
        {
            return await _goodsService.AddGoodsUint(dto);
        }
        /// <summary>
        /// 商品配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/getgoodsbompage")]
        public async Task<PageUtil> GetGoodsBomPageAsync(OrtherGoodsQueryDto dto)
        {
            return await _goodsService.GetGoodsBomPageAsync(dto);
        }
        /// <summary>
        /// 删除商品配方
        /// </summary>
        /// <param name="Ids"></param>
        /// <returns></returns>
        [HttpGet("/api/goods/delgoodsbom")]
        public async Task<bool> BatchDelGoodsBomAsync(string Ids)
        {
            return await _goodsService.BatchDelGoodsBomAsync(Ids);
        }
        /// <summary>
        /// 添加商品配方(包含属性关系)
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/addgoodsbomattribute")]
        public async Task<bool> AddGoodsBomAttribute(GoodsBomAttributeDto dto)
        {
            return await _goodsService.AddGoodsBomAttribute(dto);
        }
        /// <summary>
        /// 添加商品配方
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/goods/addgoodsbom")]
        public async Task<bool> AddGoodsBom(GoodsBomDto dto)
        {
            return await _goodsService.AddGoodsBom(dto);
        }
        /// <summary>
        /// 查询所有商品
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/goods/getgoodslist")]
        public async Task<List<BPA_GoodsInfo>> GetGoodsList()
        {
            return await _goodsService.GetGoodsList();
        }
    }
}
