﻿using BPA.SAAS.Manage.Application.Device.Dtos.DeviceTechnology;
using BPA.SAAS.Manage.Application.Device.Interface;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.Device;
using Microsoft.AspNetCore.Components.Forms;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.Device
{
    [ApiDescriptionSettings("Device", Tag = "设备基础工艺信息管理")]
    public class DeviceTechnologyServices: IDynamicApiController, ITransient
    {
        IDeviceTechnologyService _deviceTechnologyService;
        public DeviceTechnologyServices(IDeviceTechnologyService deviceTechnologyService)
        {
            _deviceTechnologyService= deviceTechnologyService;
        }
        /// <summary>
        /// 获取工艺基础信息列表
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/page")]
        public async Task<PageUtil> GetDeviceTechnologyPage(DeviceTechnologyPageBase inputDto)
        {
            return await _deviceTechnologyService.GetDeviceTechnologyPage(inputDto);
        }
        /// <summary>
        /// 查询所有工艺信息
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/devicetechnology/getdevicetechnology")]
        public async Task<List<DeviceTechnologyDto>> GetDeviceTechnology()
        {
            return await _deviceTechnologyService.GetDeviceTechnology();
        }
        /// <summary>
        /// 添加工艺
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/add")]
        public async Task<bool> AddDeviceTechnology(DeviceTechnologyBaseDto inputDto)
        {
            return await _deviceTechnologyService.AddDeviceTechnology(inputDto);
        }
        /// <summary>
        /// 修改工艺
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/update")]
        public async Task<bool> UpdateDeviceTechnology(DeviceTechnologyBaseDto inputDto)
        {
            return await _deviceTechnologyService.UpdateDeviceTechnology(inputDto);
        }
        /// <summary>
        /// 删除配方工艺
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/delete")]
        public async Task<bool> DeleteDeviceTechnology(List<string> ids)
        {
            return await _deviceTechnologyService.DeleteDeviceTechnology(ids);
        }
        /// <summary>
        /// 根据工艺id查询工艺模型
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/devicetechnology/gettechnologyaction")]
        public async Task<List<BPA_DeviceTechnologyAction>> GetTechnologyActionList(string devicetechnologyId)
        {
            return await _deviceTechnologyService.GetTechnologyActionList(devicetechnologyId);
        }
        /// <summary>
        /// 查询所有工艺模型
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/devicetechnology/gettechnologyactionlist")]
        public async Task<List<BPA_DeviceTechnologyAction>> GetTechnologyActionList()
        {
            return await _deviceTechnologyService.GetTechnologyActionList();
        }
        /// <summary>
        /// 添加工艺模型
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/adddevicetechnologyaction")]
        public async Task<bool> AddDeviceTechnologyAction(DeviceTechnologyActionBaseDto inputDto)
        {
            return await _deviceTechnologyService.AddDeviceTechnologyAction(inputDto);
        }
        /// <summary>
        /// 修改工艺模型
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/updatedevicetechnologyaction")]
        public async Task<bool> UpdateBomTechnology(DeviceTechnologyActionBaseDto inputDto)
        {
            return await _deviceTechnologyService.UpdateBomTechnology(inputDto);
        }
        /// <summary>
        /// 删除工艺模型
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpPost("/api/devicetechnology/deldevicetechnologyaction")]
        public async Task<bool> DeleteTechnologyAction(List<string> ids)
        {
            return await _deviceTechnologyService.DeleteTechnologyAction(ids);
        }
    }
}
